@@ -29,16 +29,16 @@
 #include "pango-font.h"
 #include "pango-features.h"
 #include "pango-impl-utils.h"
-#include "pango-utils-internal.h"
-#include "pango-utils-private.h"
 
 #include <glib/gstdio.h>
 
-#ifdef G_OS_WIN32
+#ifndef HAVE_FLOCKFILE
+#  define flockfile(f) (void)1
+#  define funlockfile(f) (void)1
+#  define getc_unlocked(f) getc(f)
+#endif /* !HAVE_FLOCKFILE */
 
-#define flockfile(f) (void)1
-#define funlockfile(f) (void)1
-#define getc_unlocked(f) getc(f)
+#ifdef G_OS_WIN32
 
 #include <sys/types.h>
 
@@ -50,16 +50,18 @@
 /**
  * pango_version:
  *
- * Returns the encoded version of Pango available at run-time.
+ * This is similar to the macro %PANGO_VERSION except that
+ * it returns the encoded version of Pango available at run-time,
+ * as opposed to the version available at compile-time.
  *
- * This is similar to the macro %PANGO_VERSION except that the macro
- * returns the encoded version available at compile-time. A version
- * number can be encoded into an integer using PANGO_VERSION_ENCODE().
+ * A version number can be encoded into an integer using
+ * PANGO_VERSION_ENCODE().
  *
- * Returns: The encoded version of Pango library available at run time.
+ * Returns: The encoded version of Pango library
+ *   available at run time.
  *
  * Since: 1.16
- */
+ **/
 int
 pango_version (void)
 {
@@ -69,17 +71,17 @@ pango_version (void)
 /**
  * pango_version_string:
  *
- * Returns the version of Pango available at run-time.
+ * This is similar to the macro %PANGO_VERSION_STRING except that
+ * it returns the version of Pango available at run-time, as opposed to
+ * the version available at compile-time.
  *
- * This is similar to the macro %PANGO_VERSION_STRING except that the
- * macro returns the version available at compile-time.
- *
- * Returns: (transfer none): A string containing the version of Pango library available
- *   at run time. The returned string is owned by Pango and should not
- *   be modified or freed.
+ * Returns: A string containing the version of Pango library
+ *   available at run time.
+ *   The returned string is owned by Pango and should not be modified
+ *   or freed.
  *
  * Since: 1.16
- */
+ **/
 const char *
 pango_version_string (void)
 {
@@ -88,18 +90,16 @@ pango_version_string (void)
 
 /**
  * pango_version_check:
- * @required_major: the required major version
- * @required_minor: the required minor version
- * @required_micro: the required major version
+ * @required_major: the required major version.
+ * @required_minor: the required minor version.
+ * @required_micro: the required major version.
  *
  * Checks that the Pango library in use is compatible with the
- * given version.
- *
- * Generally you would pass in the constants %PANGO_VERSION_MAJOR,
- * %PANGO_VERSION_MINOR, %PANGO_VERSION_MICRO as the three arguments
- * to this function; that produces a check that the library in use at
- * run-time is compatible with the version of Pango the application or
- * module was compiled against.
+ * given version. Generally you would pass in the constants
+ * %PANGO_VERSION_MAJOR, %PANGO_VERSION_MINOR, %PANGO_VERSION_MICRO
+ * as the three arguments to this function; that produces
+ * a check that the library in use at run-time is compatible with
+ * the version of Pango the application or module was compiled against.
  *
  * Compatibility is defined by two things: first the version
  * of the running library is newer than the version
@@ -110,13 +110,13 @@ pango_version_string (void)
  *
  * For compile-time version checking use PANGO_VERSION_CHECK().
  *
- * Returns: (transfer none) (nullable): %NULL if the Pango library is compatible
- *   with the given version, or a string describing the version
- *   mismatch.  The returned string is owned by Pango and should not
- *   be modified or freed.
+ * Return value: %NULL if the Pango library is compatible with the
+ *   given version, or a string describing the version mismatch.
+ *   The returned string is owned by Pango and should not be modified
+ *   or freed.
  *
  * Since: 1.16
- */
+ **/
 const gchar*
 pango_version_check (int required_major,
 		     int required_minor,
@@ -125,8 +125,6 @@ pango_version_check (int required_major,
   gint pango_effective_micro = 100 * PANGO_VERSION_MINOR + PANGO_VERSION_MICRO;
   gint required_effective_micro = 100 * required_minor + required_micro;
 
-  if (required_major > PANGO_VERSION_MAJOR)
-    return "Pango version too old (major mismatch)";
   if (required_major < PANGO_VERSION_MAJOR)
     return "Pango version too new (major mismatch)";
   if (required_effective_micro < pango_effective_micro - PANGO_BINARY_AGE)
@@ -142,18 +140,10 @@ pango_version_check (int required_major,
  *
  * Trims leading and trailing whitespace from a string.
  *
- * Returns: (transfer full): A newly-allocated string that must be freed with g_free()
- *
- * Deprecated: 1.38
- */
+ * Return value: A newly-allocated string that must be freed with g_free()
+ **/
 char *
 pango_trim_string (const char *str)
-{
-  return _pango_trim_string (str);
-}
-
-char *
-_pango_trim_string (const char *str)
 {
   int len;
 
@@ -176,11 +166,9 @@ _pango_trim_string (const char *str)
  * Splits a %G_SEARCHPATH_SEPARATOR-separated list of files, stripping
  * white space and substituting ~/ with $HOME/.
  *
- * Returns: (transfer full) (array zero-terminated=1): a list of
- *   strings to be freed with g_strfreev()
- *
- * Deprecated: 1.38
- */
+ * Return value: (transfer full) (array zero-terminated=1): a list of
+ * strings to be freed with g_strfreev()
+ **/
 char **
 pango_split_file_list (const char *str)
 {
@@ -192,7 +180,7 @@ pango_split_file_list (const char *str)
 
   while (files[i])
     {
-      char *file = _pango_trim_string (files[i]);
+      char *file = pango_trim_string (files[i]);
 
       /* If the resulting file is empty, skip it */
       if (file[0] == '\0')
@@ -238,23 +226,20 @@ pango_split_file_list (const char *str)
 /**
  * pango_read_line:
  * @stream: a stdio stream
- * @str: `GString` buffer into which to write the result
+ * @str: (out): #GString buffer into which to write the result
  *
- * Reads an entire line from a file into a buffer.
- *
- * Lines may be delimited with '\n', '\r', '\n\r', or '\r\n'. The delimiter
+ * Reads an entire line from a file into a buffer. Lines may
+ * be delimited with '\n', '\r', '\n\r', or '\r\n'. The delimiter
  * is not written into the buffer. Text after a '#' character is treated as
  * a comment and skipped. '\' can be used to escape a # character.
  * '\' proceeding a line delimiter combines adjacent lines. A '\' proceeding
  * any other character is ignored and written into the output buffer
  * unmodified.
  *
- * Returns: 0 if the stream was already at an %EOF character,
- *   otherwise the number of lines read (this is useful for maintaining
- *   a line number counter which doesn't combine lines with '\')
- *
- * Deprecated: 1.38
- */
+ * Return value: 0 if the stream was already at an %EOF character, otherwise
+ *               the number of lines read (this is useful for maintaining
+ *               a line number counter which doesn't combine lines with '\')
+ **/
 gint
 pango_read_line (FILE *stream, GString *str)
 {
@@ -322,12 +307,11 @@ pango_read_line (FILE *stream, GString *str)
 	      if (!comment)
 		quoted = TRUE;
 	      break;
-	    case '\r':
 	    case '\n':
 	      {
 		int next_c = getc_unlocked (stream);
 
-		if (!(next_c == EOF ||
+		if (!(c == EOF ||
 		      (c == '\r' && next_c == '\n') ||
 		      (c == '\n' && next_c == '\r')))
 		  ungetc (next_c, stream);
@@ -350,15 +334,13 @@ pango_read_line (FILE *stream, GString *str)
 
 /**
  * pango_skip_space:
- * @pos: (inout) (transfer none): in/out string position
+ * @pos: (inout): in/out string position
  *
  * Skips 0 or more characters of white space.
  *
- * Returns: %FALSE if skipping the white space leaves
- *   the position at a '\0' character.
- *
- * Deprecated: 1.38
- */
+ * Return value: %FALSE if skipping the white space leaves
+ * the position at a '\0' character.
+ **/
 gboolean
 pango_skip_space (const char **pos)
 {
@@ -374,18 +356,15 @@ pango_skip_space (const char **pos)
 
 /**
  * pango_scan_word:
- * @pos: (inout) (transfer none): in/out string position
- * @out: a `GString` into which to write the result
+ * @pos: (inout): in/out string position
+ * @out: (out): a #GString into which to write the result
  *
- * Scans a word into a `GString` buffer.
+ * Scans a word into a #GString buffer. A word consists
+ * of [A-Za-z_] followed by zero or more [A-Za-z_0-9]
+ * Leading white space is skipped.
  *
- * A word consists of [A-Za-z_] followed by zero or more
- * [A-Za-z_0-9]. Leading white space is skipped.
- *
- * Returns: %FALSE if a parse error occurred
- *
- * Deprecated: 1.38
- */
+ * Return value: %FALSE if a parse error occurred.
+ **/
 gboolean
 pango_scan_word (const char **pos, GString *out)
 {
@@ -419,19 +398,16 @@ pango_scan_word (const char **pos, GString *out)
 
 /**
  * pango_scan_string:
- * @pos: (inout) (transfer none): in/out string position
- * @out: a `GString` into which to write the result
+ * @pos: (inout): in/out string position
+ * @out: (out): a #GString into which to write the result
  *
- * Scans a string into a `GString` buffer.
- *
- * The string may either be a sequence of non-white-space characters,
- * or a quoted string with '"'. Instead a quoted string, '\"' represents
+ * Scans a string into a #GString buffer. The string may either
+ * be a sequence of non-white-space characters, or a quoted
+ * string with '"'. Instead a quoted string, '\"' represents
  * a literal quote. Leading white space outside of quotes is skipped.
  *
- * Returns: %FALSE if a parse error occurred
- *
- * Deprecated: 1.38
- */
+ * Return value: %FALSE if a parse error occurred.
+ **/
 gboolean
 pango_scan_string (const char **pos, GString *out)
 {
@@ -512,25 +488,16 @@ pango_scan_string (const char **pos, GString *out)
 
 /**
  * pango_scan_int:
- * @pos: (inout) (transfer none): in/out string position
+ * @pos: (inout): in/out string position
  * @out: (out): an int into which to write the result
  *
  * Scans an integer.
- *
  * Leading white space is skipped.
  *
- * Returns: %FALSE if a parse error occurred
- *
- * Deprecated: 1.38
- */
+ * Return value: %FALSE if a parse error occurred.
+ **/
 gboolean
 pango_scan_int (const char **pos, int *out)
-{
-  return _pango_scan_int (pos, out);
-}
-
-gboolean
-_pango_scan_int (const char **pos, int *out)
 {
   char *end;
   long temp;
@@ -554,63 +521,219 @@ _pango_scan_int (const char **pos, int *out)
   return TRUE;
 }
 
+static GHashTable *config_hash = NULL;
+static gboolean did_read_user_config = FALSE;
+
+static void
+read_config_file (const char *filename, gboolean enoent_error, GHashTable *ht)
+{
+  GKeyFile *key_file = g_key_file_new();
+  GError *key_file_error = NULL;
+  gchar **groups;
+  gsize groups_count = 0;
+  guint group_index;
+
+  if (!g_key_file_load_from_file(key_file,filename, 0, &key_file_error))
+    {
+      if (key_file_error)
+	{
+	  if (key_file_error->domain != G_FILE_ERROR || key_file_error->code != G_FILE_ERROR_NOENT || enoent_error)
+	    {
+	      g_warning ("error opening config file '%s': %s\n",
+			  filename, key_file_error->message);
+	    }
+	  g_error_free(key_file_error);
+	}
+      g_key_file_free(key_file);
+      return;
+    }
+
+  groups = g_key_file_get_groups (key_file, &groups_count);
+  for (group_index = 0; group_index < groups_count; group_index++)
+    {
+      gsize keys_count = 0;
+      const gchar *group = groups[group_index];
+      GError *keys_error = NULL;
+      gchar **keys;
+
+      keys = g_key_file_get_keys(key_file, group, &keys_count, &keys_error);
+
+      if (keys)
+	{
+	  guint key_index;
+
+	  for (key_index = 0; key_index < keys_count; key_index++)
+	    {
+	      const gchar *key = keys[key_index];
+	      GError *key_error = NULL;
+	      gchar *value =  g_key_file_get_value(key_file, group, key, &key_error);
+	      if (value != NULL)
+		{
+		  g_hash_table_insert (ht,
+				       g_strdup_printf ("%s/%s", group, key),
+				       value);
+		}
+	      if (key_error)
+		{
+		  g_warning ("error getting key '%s/%s' in config file '%s'\n",
+			     group, key, filename);
+		  g_error_free(key_error);
+		}
+	    }
+	  g_strfreev(keys);
+	}
+
+      if (keys_error)
+	{
+	  g_warning ("error getting keys in group '%s' of config file '%s'\n",
+		     filename, group);
+	  g_error_free(keys_error);
+	}
+    }
+  g_strfreev(groups);
+  g_key_file_free(key_file);
+}
+
+static void
+ensure_config_hash (void)
+{
+  if (!config_hash)
+    config_hash = g_hash_table_new_full (g_str_hash, g_str_equal,
+					 (GDestroyNotify)g_free,
+					 (GDestroyNotify)g_free);
+}
+
+static void
+read_config_system (void)
+{
+  char *filename;
+
+  ensure_config_hash ();
+
+  filename = g_build_filename (pango_get_sysconf_subdirectory (),
+			       "pangorc",
+			       NULL);
+  read_config_file (filename, FALSE, config_hash);
+  g_free (filename);
+}
+
+static void
+read_config (void)
+{
+  char *filename;
+  const char *envvar;
+
+  read_config_system ();
+
+  if (!did_read_user_config)
+    {
+      did_read_user_config = TRUE;
+
+      filename = g_build_filename (g_get_user_config_dir (), "pango", "pangorc", NULL);
+      read_config_file (filename, FALSE, config_hash);
+      g_free (filename);
+
+      envvar = g_getenv ("PANGO_RC_FILE");
+      if (envvar)
+        read_config_file (envvar, TRUE, config_hash);
+    }
+}
+
 /**
  * pango_config_key_get_system:
- * @key: Key to look up, in the form "SECTION/KEY"
+ * @key: Key to look up, in the form "SECTION/KEY".
  *
- * Do not use.  Does not do anything.
+ * Looks up a key, consulting only the Pango system config database
+ * in $sysconfdir/pango/pangorc.
  *
- * Returns: (nullable): %NULL
- *
- * Deprecated: 1.38
- */
+ * Return value: the value, if found, otherwise %NULL. The value is a
+ * newly-allocated string and must be freed with g_free().
+ **/
 char *
 pango_config_key_get_system (const char *key)
 {
-  return NULL;
+  g_return_val_if_fail (key != NULL, NULL);
+
+  read_config_system ();
+
+  return g_strdup (g_hash_table_lookup (config_hash, key));
 }
 
 /**
  * pango_config_key_get:
- * @key: Key to look up, in the form "SECTION/KEY"
+ * @key: Key to look up, in the form "SECTION/KEY".
  *
- * Do not use.  Does not do anything.
+ * Looks up a key in the Pango config database
+ * (pseudo-win.ini style, read from $sysconfdir/pango/pangorc,
+ *  $XDG_CONFIG_HOME/pango/pangorc, and getenv (PANGO_RC_FILE).)
  *
- * Returns: (nullable): %NULL
- *
- * Deprecated: 1.38
- */
+ * Return value: the value, if found, otherwise %NULL. The value is a
+ * newly-allocated string and must be freed with g_free().
+ **/
 char *
 pango_config_key_get (const char *key)
 {
-  return NULL;
+  g_return_val_if_fail (key != NULL, NULL);
+
+  read_config ();
+
+  return g_strdup (g_hash_table_lookup (config_hash, key));
 }
 
+#ifdef G_OS_WIN32
+
+/* DllMain function needed to tuck away the DLL handle */
+
+static HMODULE pango_dll; /* MT-safe */
+
+BOOL WINAPI
+DllMain (HINSTANCE hinstDLL,
+	 DWORD     fdwReason,
+	 LPVOID    lpvReserved)
+{
+  switch (fdwReason)
+    {
+    case DLL_PROCESS_ATTACH:
+      pango_dll = (HMODULE) hinstDLL;
+      break;
+    }
+
+  return TRUE;
+}
+
+#endif
+
 /**
  * pango_get_sysconf_subdirectory:
  *
- * Returns the name of the "pango" subdirectory of SYSCONFDIR
- * (which is set at compile time).
+ * On Unix, returns the name of the "pango" subdirectory of SYSCONFDIR
+ * (which is set at compile time). On Windows, returns the etc\pango
+ * subdirectory of the Pango installation directory (which is deduced
+ * at run time from the DLL's location).
  *
- * Returns: (transfer none): the Pango sysconf directory. The returned string should
+ * Return value: the Pango sysconf directory. The returned string should
  * not be freed.
- *
- * Deprecated: 1.38
  */
 const char *
 pango_get_sysconf_subdirectory (void)
 {
   static const gchar *result = NULL; /* MT-safe */
 
-  if (g_once_init_enter (&result))
+  if (g_once_init_enter ((gsize*)&result))
     {
       const char *tmp_result = NULL;
+#ifdef G_OS_WIN32
+      gchar *root = g_win32_get_package_installation_directory_of_module (pango_dll);
+      tmp_result = g_build_filename (root, "etc\\pango", NULL);
+      g_free (root);
+#else
       const char *sysconfdir = g_getenv ("PANGO_SYSCONFDIR");
       if (sysconfdir != NULL)
 	tmp_result = g_build_filename (sysconfdir, "pango", NULL);
       else
 	tmp_result = SYSCONFDIR "/pango";
-      g_once_init_leave(&result, tmp_result);
+#endif
+      g_once_init_leave((gsize*)&result, (gsize)tmp_result);
     }
   return result;
 }
@@ -618,28 +741,40 @@ pango_get_sysconf_subdirectory (void)
 /**
  * pango_get_lib_subdirectory:
  *
- * Returns the name of the "pango" subdirectory of LIBDIR
- * (which is set at compile time).
+ * On Unix, returns the name of the "pango" subdirectory of LIBDIR
+ * (which is set at compile time). On Windows, returns the lib\pango
+ * subdirectory of the Pango installation directory (which is deduced
+ * at run time from the DLL's location).
  *
- * Returns: (transfer none): the Pango lib directory. The returned string should
+ * Return value: the Pango lib directory. The returned string should
  * not be freed.
- *
- * Deprecated: 1.38
  */
 const char *
 pango_get_lib_subdirectory (void)
 {
   static const gchar *result = NULL; /* MT-safe */
 
-  if (g_once_init_enter (&result))
+  if (g_once_init_enter ((gsize*)&result))
     {
       const gchar *tmp_result = NULL;
+#ifdef G_OS_WIN32
+      gchar *root = g_win32_get_package_installation_directory_of_module (pango_dll);
+      /* If we are running against an uninstalled copy of the Pango DLL,
+       * use the compile-time installation prefix.
+       */
+      if (g_str_has_suffix (root, "\\.libs"))
+	tmp_result = g_strdup (LIBDIR "/pango");
+      else
+	tmp_result = g_build_filename (root, "lib\\pango", NULL);
+      g_free (root);
+#else
       const char *libdir = g_getenv ("PANGO_LIBDIR");
       if (libdir != NULL)
 	tmp_result = g_build_filename (libdir, "pango", NULL);
       else
 	tmp_result = LIBDIR "/pango";
-      g_once_init_leave(&result, tmp_result);
+#endif
+      g_once_init_leave((gsize*)&result, (gsize)tmp_result);
     }
   return result;
 }
@@ -650,14 +785,8 @@ parse_int (const char *word,
 	   int        *out)
 {
   char *end;
-  long val;
-  int i;
-
-  if (word == NULL)
-    return FALSE;
-
-  val = strtol (word, &end, 10);
-  i = val;
+  long val = strtol (word, &end, 10);
+  int i = val;
 
   if (end != word && *end == '\0' && val >= 0 && val == i)
     {
@@ -672,46 +801,32 @@ parse_int (const char *word,
 
 /**
  * pango_parse_enum:
- * @type: enum type to parse, eg. %PANGO_TYPE_ELLIPSIZE_MODE
- * @str: (nullable): string to parse
- * @value: (out) (optional): integer to store the result in
- * @warn: if %TRUE, issue a g_warning() on bad input
- * @possible_values: (out) (optional): place to store list of possible
- *   values on failure
+ * @type: enum type to parse, eg. %PANGO_TYPE_ELLIPSIZE_MODE.
+ * @str: (allow-none): string to parse.  May be %NULL.
+ * @value: (out) (allow-none): integer to store the result in, or %NULL.
+ * @warn: if %TRUE, issue a g_warning() on bad input.
+ * @possible_values: (out) (allow-none): place to store list of possible values on failure, or %NULL.
  *
  * Parses an enum type and stores the result in @value.
  *
- * If @str does not match the nick name of any of the possible values
- * for the enum and is not an integer, %FALSE is returned, a warning
- * is issued if @warn is %TRUE, and a string representing the list of
- * possible values is stored in @possible_values. The list is
- * slash-separated, eg. "none/start/middle/end".
+ * If @str does not match the nick name of any of the possible values for the
+ * enum and is not an integer, %FALSE is returned, a warning is issued
+ * if @warn is %TRUE, and a
+ * string representing the list of possible values is stored in
+ * @possible_values.  The list is slash-separated, eg.
+ * "none/start/middle/end".  If failed and @possible_values is not %NULL,
+ * returned string should be freed using g_free().
  *
- * If failed and @possible_values is not %NULL, returned string should
- * be freed using g_free().
- *
- * Returns: %TRUE if @str was successfully parsed
- *
- * Deprecated: 1.38
+ * Return value: %TRUE if @str was successfully parsed.
  *
  * Since: 1.16
- */
+ **/
 gboolean
 pango_parse_enum (GType       type,
 		  const char *str,
 		  int        *value,
 		  gboolean    warn,
 		  char      **possible_values)
-{
-  return _pango_parse_enum (type, str, value, warn, possible_values);
-}
-
-gboolean
-_pango_parse_enum (GType       type,
-		   const char *str,
- 		   int        *value,
-		   gboolean    warn,
-		   char      **possible_values)
 {
   GEnumClass *class = NULL;
   gboolean ret = TRUE;
@@ -749,69 +864,9 @@ _pango_parse_enum (GType       type,
 		       s->str);
 
 	  if (possible_values)
-	    *possible_values = g_string_free (s, FALSE);
-          else
-	    g_string_free (s, TRUE);
-	}
-    }
+	    *possible_values = s->str;
 
-  g_type_class_unref (class);
-
-  return ret;
-}
-
-gboolean
-pango_parse_flags (GType        type,
-                   const char  *str,
-                   int         *value,
-                   char       **possible_values)
-{
-  GFlagsClass *class = NULL;
-  gboolean ret = TRUE;
-  GFlagsValue *v = NULL;
-
-  class = g_type_class_ref (type);
-
-  v = g_flags_get_value_by_nick (class, str);
-
-  if (v)
-    {
-      *value = v->value;
-    }
-  else if (!parse_int (str, value))
-    {
-      char **strv = g_strsplit (str, "|", 0);
-      int i;
-
-      *value = 0;
-
-      for (i = 0; strv[i]; i++)
-        {
-          strv[i] = g_strstrip (strv[i]);
-          v = g_flags_get_value_by_nick (class, strv[i]);
-          if (!v)
-            {
-              ret = FALSE;
-              break;
-            }
-          *value |= v->value;
-        }
-      g_strfreev (strv);
-
-      if (!ret && possible_values)
-	{
-	  int i;
-	  GString *s = g_string_new (NULL);
-
-          for (i = 0; i < class->n_values; i++)
-            {
-              v = &class->values[i];
-              if (i)
-                g_string_append_c (s, '/');
-              g_string_append (s, v->value_nick);
-            }
-
-          *possible_values = g_string_free (s, FALSE);
+	  g_string_free (s, possible_values ? FALSE : TRUE);
 	}
     }
 
@@ -822,13 +877,12 @@ pango_parse_flags (GType        type,
 
 /**
  * pango_lookup_aliases:
- * @fontname: an ASCII string
- * @families: (out) (array length=n_families): will be set to an array of
- *   font family names. This array is owned by Pango and should not be freed
- * @n_families: (out): will be set to the length of the @families array
+ * @fontname: an ascii string
+ * @families: (out) (array length=n_families): will be set to an array of font family names.
+ *    this array is owned by pango and should not be freed.
+ * @n_families: (out): will be set to the length of the @families array.
  *
  * Look up all user defined aliases for the alias @fontname.
- *
  * The resulting font family names will be stored in @families,
  * and the number of families in @n_families.
  *
@@ -843,19 +897,16 @@ pango_lookup_aliases (const char   *fontname,
   *n_families = 0;
 }
 
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
-
 /**
  * pango_find_base_dir:
- * @text: the text to process. Must be valid UTF-8
+ * @text:   the text to process
  * @length: length of @text in bytes (may be -1 if @text is nul-terminated)
  *
  * Searches a string the first character that has a strong
  * direction, according to the Unicode bidirectional algorithm.
  *
- * Returns: The direction corresponding to the first strong character.
- *   If no such character is found, then %PANGO_DIRECTION_NEUTRAL is returned.
+ * Return value: The direction corresponding to the first strong character.
+ * If no such character is found, then %PANGO_DIRECTION_NEUTRAL is returned.
  *
  * Since: 1.4
  */
@@ -884,20 +935,17 @@ pango_find_base_dir (const gchar *text,
   return dir;
 }
 
-#pragma GCC diagnostic pop
-
 /**
  * pango_is_zero_width:
  * @ch: a Unicode character
  *
- * Checks if a character that should not be normally rendered.
+ * Checks @ch to see if it is a character that should not be
+ * normally rendered on the screen.  This includes all Unicode characters
+ * with "ZERO WIDTH" in their name, as well as <firstterm>bidi</firstterm> formatting characters, and
+ * a few other ones.  This is totally different from g_unichar_iszerowidth()
+ * and is at best misnamed.
  *
- * This includes all Unicode characters with "ZERO WIDTH" in their name,
- * as well as *bidi* formatting characters, and a few other ones.
- *
- * This is totally different from [func@GLib.unichar_iszerowidth] and is at best misnamed.
- *
- * Returns: %TRUE if @ch is a zero-width character, %FALSE otherwise
+ * Return value: %TRUE if @ch is a zero-width character, %FALSE otherwise
  *
  * Since: 1.10
  */
@@ -917,29 +965,23 @@ pango_is_zero_width (gunichar ch)
  *
  *  2028  LINE SEPARATOR
  *
- *  2060  WORD JOINER
- *  2061  FUNCTION APPLICATION
- *  2062  INVISIBLE TIMES
- *  2063  INVISIBLE SEPARATOR
- *
- *  2066  LEFT-TO-RIGHT ISOLATE
- *  2067  RIGHT-TO-LEFT ISOLATE
- *  2068  FIRST STRONG ISOLATE
- *  2069  POP DIRECTIONAL ISOLATE
- *
  *  202A  LEFT-TO-RIGHT EMBEDDING
  *  202B  RIGHT-TO-LEFT EMBEDDING
  *  202C  POP DIRECTIONAL FORMATTING
  *  202D  LEFT-TO-RIGHT OVERRIDE
  *  202E  RIGHT-TO-LEFT OVERRIDE
  *
+ *  2060  WORD JOINER
+ *  2061  FUNCTION APPLICATION
+ *  2062  INVISIBLE TIMES
+ *  2063  INVISIBLE SEPARATOR
+ *
  *  FEFF  ZERO WIDTH NO-BREAK SPACE
  */
   return ((ch & ~(gunichar)0x007F) == 0x2000 && (
 		(ch >= 0x200B && ch <= 0x200F) ||
 		(ch >= 0x202A && ch <= 0x202E) ||
 		(ch >= 0x2060 && ch <= 0x2063) ||
-                (ch >= 0x2066 && ch <= 0x2069) ||
 		(ch == 0x2028)
 	 )) || G_UNLIKELY (ch == 0x00AD
 			|| ch == 0x034F
@@ -951,10 +993,10 @@ pango_is_zero_width (gunichar ch)
  * @thickness: (inout): pointer to the thickness of a line, in Pango units
  * @position: (inout): corresponding position
  *
- * Quantizes the thickness and position of a line to whole device pixels.
- *
- * This is typically used for underline or strikethrough. The purpose of
- * this function is to avoid such lines looking blurry.
+ * Quantizes the thickness and position of a line, typically an
+ * underline or strikethrough, to whole device pixels, that is integer
+ * multiples of %PANGO_SCALE. The purpose of this function is to avoid
+ * such lines looking blurry.
  *
  * Care is taken to make sure @thickness is at least one pixel when this
  * function returns, but returned @position may become zero as a result
@@ -988,12 +1030,10 @@ pango_quantize_line_geometry (int *thickness,
  * pango_units_from_double:
  * @d: double floating-point value
  *
- * Converts a floating-point number to Pango units.
+ * Converts a floating-point number to Pango units: multiplies
+ * it by %PANGO_SCALE and rounds to nearest integer.
  *
- * The conversion is done by multiplying @d by %PANGO_SCALE and
- * rounding the result to nearest integer.
- *
- * Returns: the value in Pango units.
+ * Return value: the value in Pango units.
  *
  * Since: 1.16
  */
@@ -1007,11 +1047,10 @@ pango_units_from_double (double d)
  * pango_units_to_double:
  * @i: value in Pango units
  *
- * Converts a number in Pango units to floating-point.
+ * Converts a number in Pango units to floating-point: divides
+ * it by %PANGO_SCALE.
  *
- * The conversion is done by dividing @i by %PANGO_SCALE.
- *
- * Returns: the double value.
+ * Return value: the double value.
  *
  * Since: 1.16
  */
@@ -1023,29 +1062,27 @@ pango_units_to_double (int i)
 
 /**
  * pango_extents_to_pixels:
- * @inclusive: (nullable): rectangle to round to pixels inclusively
- * @nearest: (nullable): rectangle to round to nearest pixels
+ * @inclusive: (allow-none): rectangle to round to pixels inclusively, or %NULL.
+ * @nearest: (allow-none): rectangle to round to nearest pixels, or %NULL.
  *
- * Converts extents from Pango units to device units.
+ * Converts extents from Pango units to device units, dividing by the
+ * %PANGO_SCALE factor and performing rounding.
  *
- * The conversion is done by dividing by the %PANGO_SCALE factor and
- * performing rounding.
- *
- * The @inclusive rectangle is converted by flooring the x/y coordinates
- * and extending width/height, such that the final rectangle completely
- * includes the original rectangle.
+ * The @inclusive rectangle is converted by flooring the x/y coordinates and extending
+ * width/height, such that the final rectangle completely includes the original
+ * rectangle.
  *
  * The @nearest rectangle is converted by rounding the coordinates
  * of the rectangle to the nearest device unit (pixel).
  *
  * The rule to which argument to use is: if you want the resulting device-space
- * rectangle to completely contain the original rectangle, pass it in as
- * @inclusive. If you want two touching-but-not-overlapping rectangles stay
+ * rectangle to completely contain the original rectangle, pass it in as @inclusive.
+ * If you want two touching-but-not-overlapping rectangles stay
  * touching-but-not-overlapping after rounding to device units, pass them in
  * as @nearest.
  *
  * Since: 1.16
- */
+ **/
 void
 pango_extents_to_pixels (PangoRectangle *inclusive,
 			 PangoRectangle *nearest)
@@ -1085,12 +1122,12 @@ pango_extents_to_pixels (PangoRectangle *inclusive,
 
 void
 _pango_shape_shape (const char       *text,
-		    unsigned int      n_chars,
+		    gint              n_chars,
 		    PangoRectangle   *shape_ink G_GNUC_UNUSED,
 		    PangoRectangle   *shape_logical,
 		    PangoGlyphString *glyphs)
 {
-  unsigned int i;
+  int i;
   const char *p;
 
   pango_glyph_string_set_size (glyphs, n_chars);
@@ -1151,105 +1188,3 @@ _pango_shape_get_extents (gint              n_chars,
     }
 }
 
-/**
- * pango_find_paragraph_boundary:
- * @text: UTF-8 text
- * @length: length of @text in bytes, or -1 if nul-terminated
- * @paragraph_delimiter_index: (out): return location for index of
- *   delimiter
- * @next_paragraph_start: (out): return location for start of next
- *   paragraph
- *
- * Locates a paragraph boundary in @text.
- *
- * A boundary is caused by delimiter characters, such as
- * a newline, carriage return, carriage return-newline pair,
- * or Unicode paragraph separator character.
- *
- * The index of the run of delimiters is returned in
- * @paragraph_delimiter_index. The index of the start of the
- * next paragraph (index after all delimiters) is stored n
- * @next_paragraph_start.
- *
- * If no delimiters are found, both @paragraph_delimiter_index
- * and @next_paragraph_start are filled with the length of @text
- * (an index one off the end).
- */
-void
-pango_find_paragraph_boundary (const char *text,
-                               int         length,
-                               int        *paragraph_delimiter_index,
-                               int        *next_paragraph_start)
-{
-  const char *p = text;
-  const char *end;
-  const char *start = NULL;
-  const char *delimiter = NULL;
-
-  /* Only one character has type G_UNICODE_PARAGRAPH_SEPARATOR in
-   * Unicode 5.0; update the following code if that changes.
-   */
-
-  /* prev_sep is the first byte of the previous separator.  Since
-   * the valid separators are \r, \n, and PARAGRAPH_SEPARATOR, the
-   * first byte is enough to identify it.
-   */
-  char prev_sep;
-
-#define PARAGRAPH_SEPARATOR_STRING "\xE2\x80\xA9"
-
-  if (length < 0)
-    length = strlen (text);
-
-  end = text + length;
-
-  if (paragraph_delimiter_index)
-    *paragraph_delimiter_index = length;
-
-  if (next_paragraph_start)
-    *next_paragraph_start = length;
-
-  if (length == 0)
-    return;
-
-  prev_sep = 0;
-  while (p < end)
-    {
-      if (prev_sep == '\n' ||
-          prev_sep == PARAGRAPH_SEPARATOR_STRING[0])
-        {
-          g_assert (delimiter);
-          start = p;
-          break;
-        }
-      else if (prev_sep == '\r')
-        {
-          /* don't break between \r and \n */
-          if (*p != '\n')
-            {
-              g_assert (delimiter);
-              start = p;
-              break;
-            }
-        }
-
-      if (*p == '\n' ||
-           *p == '\r' ||
-           !strncmp(p, PARAGRAPH_SEPARATOR_STRING, strlen (PARAGRAPH_SEPARATOR_STRING)))
-        {
-          if (delimiter == NULL)
-            delimiter = p;
-          prev_sep = *p;
-        }
-      else
-        prev_sep = 0;
-
-      p = g_utf8_next_char (p);
-    }
-
-  if (delimiter && paragraph_delimiter_index)
-    *paragraph_delimiter_index = delimiter - text;
-
-  if (start && next_paragraph_start)
-    *next_paragraph_start = start - text;
-}
