@@ -29,14 +29,13 @@
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
-#include <glib/gstdio.h>
 
 #include "pango-fontmap.h"
 #include "pango-impl-utils.h"
 #include "pangowin32-private.h"
+#include "modules.h"
 
-typedef struct _PangoWin32Family PangoWin32Family;
-typedef PangoFontFamilyClass PangoWin32FamilyClass;
+typedef struct _PangoWin32Family       PangoWin32Family;
 
 struct _PangoWin32Family
 {
@@ -48,7 +47,6 @@ struct _PangoWin32Family
   gboolean is_monospace;
 };
 
-
 #if !defined(NTM_PS_OPENTYPE)
 # define NTM_PS_OPENTYPE 0x20000
 #endif
@@ -59,17 +57,11 @@ struct _PangoWin32Family
 
 #define PANGO_WIN32_TYPE_FAMILY              (pango_win32_family_get_type ())
 #define PANGO_WIN32_FAMILY(object)           (G_TYPE_CHECK_INSTANCE_CAST ((object), PANGO_WIN32_TYPE_FAMILY, PangoWin32Family))
-#define PANGO_WIN32_IS_FAMILY (object)       (G_TYPE_CHECK_INSTANCE_TYPE ((object), PANGO_WIN32_TYPE_FAMILY))
-#define PANGO_WIN32_FAMILY_CLASS (klass)     (G_TYPE_CHECK_CLASS_CAST (klass), PANGO_WIN32_TYPE_FAMILY, PangoWin32Family))
-#define PANGO_WIN32_IS_FAMILY_CLASS (klass)  (G_TYPE_CHECK_CLASS_CAST (klass), PANGO_WIN32_TYPE_FAMILY))
-#define PANGO_WIN32_FAMILY_GET_CLASS(object) (G_TYPE_INSTANCE_GET_CLASS((object), PANGO_WIN32_FAMILY, PangoWin32FamilyClass))
+#define PANGO_WIN32_IS_FAMILY(object)        (G_TYPE_CHECK_INSTANCE_TYPE ((object), PANGO_WIN32_TYPE_FAMILY))
 
 #define PANGO_WIN32_TYPE_FACE              (pango_win32_face_get_type ())
 #define PANGO_WIN32_FACE(object)           (G_TYPE_CHECK_INSTANCE_CAST ((object), PANGO_WIN32_TYPE_FACE, PangoWin32Face))
 #define PANGO_WIN32_IS_FACE(object)        (G_TYPE_CHECK_INSTANCE_TYPE ((object), PANGO_WIN32_TYPE_FACE))
-#define PANGO_WIN32_FACE_CLASS (klass)     (G_TYPE_CHECK_CLASS_CAST (klass), PANGO_WIN32_TYPE_FACE, PangoWin32Face))
-#define PANGO_WIN32_IS_FACE_CLASS (klass)  (G_TYPE_CHECK_CLASS_CAST (klass), PANGO_WIN32_TYPE_FACE))
-#define PANGO_WIN32_FACE_GET_CLASS(object) (G_TYPE_INSTANCE_GET_CLASS((object), PANGO_WIN32_FACE, PangoWin32FaceClass))
 
 static GType      pango_win32_face_get_type          (void);
 
@@ -83,10 +75,6 @@ static void       pango_win32_font_map_finalize      (GObject
 static PangoFont *pango_win32_font_map_load_font     (PangoFontMap                 *fontmap,
 						      PangoContext                 *context,
 						      const PangoFontDescription   *description);
-static PangoFontset *pango_win32_font_map_load_fontset (PangoFontMap               *fontmap,
-						      PangoContext                 *context,
-						      const PangoFontDescription   *desc,
-						      PangoLanguage                *language);
 static void       pango_win32_font_map_list_families (PangoFontMap                 *fontmap,
 						      PangoFontFamily            ***families,
 						      int                          *n_families);
@@ -319,6 +307,8 @@ struct PangoAlias
   gboolean visible; /* Do we want/need this? */
 };
 
+static GHashTable *pango_aliases_ht = NULL; /* MT-unsafe */
+
 static guint
 alias_hash (struct PangoAlias *alias)
 {
@@ -348,9 +338,8 @@ alias_free (struct PangoAlias *alias)
 }
 
 static void
-handle_alias_line (GString    *line_buffer,
-                   char       **errstring,
-                   GHashTable *ht_aliases)
+handle_alias_line (GString  *line_buffer,
+                   char    **errstring)
 {
   GString *tmp_buffer1;
   GString *tmp_buffer2;
@@ -402,14 +391,14 @@ handle_alias_line (GString    *line_buffer,
   alias_key.alias = g_ascii_strdown (tmp_buffer1->str, -1);
 
   /* Remove any existing values */
-  alias = g_hash_table_lookup (ht_aliases, &alias_key);
+  alias = g_hash_table_lookup (pango_aliases_ht, &alias_key);
 
   if (!alias)
     {
       alias = g_slice_new0 (struct PangoAlias);
       alias->alias = alias_key.alias;
 
-      g_hash_table_insert (ht_aliases, alias, alias);
+      g_hash_table_insert (pango_aliases_ht, alias, alias);
     }
   else
     g_free (alias_key.alias);
@@ -466,7 +455,7 @@ static const char * const builtin_aliases[] = {
 };
 
 static void
-read_builtin_aliases (GHashTable *ht_aliases)
+read_builtin_aliases (void)
 {
 
   GString *line_buffer;
@@ -478,7 +467,7 @@ read_builtin_aliases (GHashTable *ht_aliases)
   for (line = 0; line < G_N_ELEMENTS (builtin_aliases) && errstring == NULL; line++)
     {
       g_string_assign (line_buffer, builtin_aliases[line]);
-      handle_alias_line (line_buffer, &errstring, ht_aliases);
+      handle_alias_line (line_buffer, &errstring);
     }
 
   if (errstring)
@@ -489,22 +478,74 @@ read_builtin_aliases (GHashTable *ht_aliases)
 
   g_string_free (line_buffer, TRUE);
 }
-#endif
 
 
-static GHashTable *
-load_aliases (void)
+static void
+read_alias_file (const char *filename)
 {
-  GHashTable *ht_aliases = g_hash_table_new_full ((GHashFunc)alias_hash,
-                                                  (GEqualFunc)alias_equal,
-                                                  (GDestroyNotify)alias_free,
-                                                  NULL);
+  FILE *file;
+
+  GString *line_buffer;
+  char *errstring = NULL;
+  int line = 0;
+
+  file = g_fopen (filename, "r");
+  if (!file)
+    return;
+
+  line_buffer = g_string_new (NULL);
+
+  while (pango_read_line (file, line_buffer) &&
+         errstring == NULL)
+    {
+      line++;
+      handle_alias_line (line_buffer, &errstring);
+    }
+
+  if (errstring == NULL && ferror (file))
+    errstring = g_strdup (g_strerror(errno));
+
+  if (errstring)
+    {
+      g_warning ("error reading alias file: %s:%d: %s\n", filename, line, errstring);
+      g_free (errstring);
+    }
+
+  g_string_free (line_buffer, TRUE);
+
+  fclose (file);
+}
+
+static void
+pango_load_aliases (void)
+{
+  char *filename;
+  const char *home;
+
+  pango_aliases_ht = g_hash_table_new_full ((GHashFunc)alias_hash,
+                                            (GEqualFunc)alias_equal,
+                                            (GDestroyNotify)alias_free,
+                                            NULL);
 
 #ifdef HAVE_CAIRO_WIN32
-  read_builtin_aliases (ht_aliases);
+  read_builtin_aliases ();
 #endif
 
-  return ht_aliases;
+  filename = g_strconcat (pango_get_sysconf_subdirectory (),
+                          G_DIR_SEPARATOR_S "pango.aliases",
+                          NULL);
+  read_alias_file (filename);
+  g_free (filename);
+
+  home = g_get_home_dir ();
+  if (home && *home)
+    {
+      filename = g_strconcat (home,
+                              G_DIR_SEPARATOR_S ".pango.aliases",
+                              NULL);
+      read_alias_file (filename);
+      g_free (filename);
+    }
 }
 
 static void
@@ -512,18 +553,14 @@ lookup_aliases (const char   *fontname,
                 char       ***families,
                 int          *n_families)
 {
-  static GHashTable *aliases_ht = NULL; /* MT-safe */
-
   struct PangoAlias alias_key;
   struct PangoAlias *alias;
 
-  if (g_once_init_enter (&aliases_ht))
-    {
-      g_once_init_leave (&aliases_ht, load_aliases ());
-    }
+  if (pango_aliases_ht == NULL)
+    load_aliases ();
 
   alias_key.alias = g_ascii_strdown (fontname, -1);
-  alias = g_hash_table_lookup (aliases_ht, &alias_key);
+  alias = g_hash_table_lookup (pango_aliases_ht, &alias_key);
   g_free (alias_key.alias);
 
   if (alias)
@@ -576,8 +613,6 @@ create_standard_family (PangoWin32FontMap *win32fontmap,
 		    new_face->coverages[j] = NULL;
 		}
 
-	      new_face->face_name = NULL;
-
 	      new_face->is_synthetic = TRUE;
 
 	      new_face->has_cmap = old_face->has_cmap;
@@ -603,12 +638,10 @@ _pango_win32_font_map_init (PangoWin32FontMap *win32fontmap)
 {
   LOGFONTW logfont;
 
-  win32fontmap->families =
-    g_hash_table_new_full ((GHashFunc) case_insensitive_str_hash,
-                           (GEqualFunc) case_insensitive_str_equal, NULL, g_object_unref);
+  win32fontmap->families = g_hash_table_new ((GHashFunc) case_insensitive_str_hash,
+					     (GEqualFunc) case_insensitive_str_equal);
   win32fontmap->fonts =
-    g_hash_table_new_full ((GHashFunc) logfontw_nosize_hash,
-                           (GEqualFunc) logfontw_nosize_equal, NULL, g_free);
+    g_hash_table_new ((GHashFunc) logfontw_nosize_hash, (GEqualFunc) logfontw_nosize_equal);
 
   win32fontmap->font_cache = pango_win32_font_cache_new ();
   win32fontmap->freed_fonts = g_queue_new ();
@@ -629,63 +662,29 @@ _pango_win32_font_map_init (PangoWin32FontMap *win32fontmap)
   win32fontmap->resolution = (PANGO_SCALE / (double) GetDeviceCaps (_pango_win32_hdc, LOGPIXELSY)) * 72.0;
 }
 
-static void
-pango_win32_font_map_fontset_add_fonts (PangoFontMap          *fontmap,
-				  PangoContext          *context,
-				  PangoFontsetSimple *fonts,
-				  PangoFontDescription  *desc,
-				  const char            *family)
-{
-  /* Mostly use the "old" pango_font_map_fontset_add_fonts() */
-  /* on Windows so that we can go through the .aliases file */
-  /* to load the appropriate fontset for various texts */
-  PangoFont *font;
-  char **aliases;
-  int n_aliases;
-  int j;
-
-  lookup_aliases (family, &aliases, &n_aliases);
-
-  if (n_aliases)
-  {
-    for (j = 0; j < n_aliases; j++)
-    {
-      pango_font_description_set_family_static (desc, aliases[j]);
-      font = pango_win32_font_map_load_font (fontmap, context, desc);
-      if (font)
-        pango_fontset_simple_append (fonts, font);
-    }
-  }
-  else
-  {
-    pango_font_description_set_family_static (desc, family);
-    font = pango_win32_font_map_load_font (fontmap, context, desc);
-    if (font)
-      pango_fontset_simple_append (fonts, font);
-  }
-}
-
 static void
 _pango_win32_font_map_class_init (PangoWin32FontMapClass *class)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (class);
   PangoFontMapClass *fontmap_class = PANGO_FONT_MAP_CLASS (class);
+  int i;
 
   class->find_font = pango_win32_font_map_real_find_font;
   object_class->finalize = pango_win32_font_map_finalize;
   fontmap_class->load_font = pango_win32_font_map_load_font;
-  /* we now need a load_fontset implementation for the Win32 backend */
-  fontmap_class->load_fontset = pango_win32_font_map_load_fontset;
   fontmap_class->list_families = pango_win32_font_map_list_families;
   fontmap_class->shape_engine_type = PANGO_RENDER_TYPE_WIN32;
 
   pango_win32_get_dc ();
+
+  for (i = 0; _pango_included_win32_modules[i].list; i++)
+    pango_module_register (&_pango_included_win32_modules[i]);
 }
 
 /**
  * pango_win32_font_map_for_display:
  *
- * Returns a <type>PangoWin32FontMap</type>. Font maps are cached and should
+ * Returns a #PangoWin32FontMap. Font maps are cached and should
  * not be freed. If the font map is no longer needed, it can
  * be released with pango_win32_shutdown_display().
  *
@@ -694,10 +693,8 @@ _pango_win32_font_map_class_init (PangoWin32FontMapClass *class)
 PangoFontMap *
 pango_win32_font_map_for_display (void)
 {
-#if !GLIB_CHECK_VERSION (2, 35, 3)
   /* Make sure that the type system is initialized */
   g_type_init ();
-#endif
 
   if (g_once_init_enter ((gsize*)&default_fontmap))
     g_once_init_leave((gsize*)&default_fontmap, (gsize)g_object_new (PANGO_TYPE_WIN32_FONT_MAP, NULL));
@@ -732,10 +729,6 @@ pango_win32_font_map_finalize (GObject *object)
 
   pango_win32_font_cache_free (win32fontmap->font_cache);
 
-  g_hash_table_destroy (win32fontmap->fonts);
-
-  g_hash_table_destroy (win32fontmap->families);
-
   G_OBJECT_CLASS (_pango_win32_font_map_parent_class)->finalize (object);
 }
 
@@ -793,34 +786,40 @@ pango_win32_family_is_monospace (PangoFontFamily *family)
   return win32family->is_monospace;
 }
 
-G_DEFINE_TYPE (PangoWin32Family, pango_win32_family, PANGO_TYPE_FONT_FAMILY)
-
-static void
-pango_win32_family_finalize (GObject *object)
-{
-  PangoWin32Family *win32family = PANGO_WIN32_FAMILY (object);
-
-  g_free (win32family->family_name);
-
-  g_slist_free_full (win32family->faces, g_object_unref);
-
-  G_OBJECT_CLASS (pango_win32_family_parent_class)->finalize (object);
-}
-
 static void
 pango_win32_family_class_init (PangoFontFamilyClass *class)
 {
-  GObjectClass *object_class = G_OBJECT_CLASS (class);
-
-  object_class->finalize = pango_win32_family_finalize;
   class->list_faces = pango_win32_family_list_faces;
   class->get_name = pango_win32_family_get_name;
   class->is_monospace = pango_win32_family_is_monospace;
 }
 
-static void
-pango_win32_family_init (PangoWin32Family *family)
+static GType
+pango_win32_family_get_type (void)
 {
+  static GType object_type = 0;
+
+  if (G_UNLIKELY (!object_type))
+    {
+      const GTypeInfo object_info =
+      {
+	sizeof (PangoFontFamilyClass),
+	(GBaseInitFunc) NULL,
+	(GBaseFinalizeFunc) NULL,
+	(GClassInitFunc) pango_win32_family_class_init,
+	NULL,           /* class_finalize */
+	NULL,           /* class_data */
+	sizeof (PangoWin32Family),
+	0,              /* n_preallocs */
+	(GInstanceInitFunc) NULL,
+      };
+
+      object_type = g_type_register_static (PANGO_TYPE_FONT_FAMILY,
+					    I_("PangoWin32Family"),
+					    &object_info, 0);
+    }
+
+  return object_type;
 }
 
 static void
@@ -944,9 +943,9 @@ pango_win32_font_neww (PangoFontMap   *fontmap,
 
   result = (PangoWin32Font *)g_object_new (PANGO_TYPE_WIN32_FONT, NULL);
 
-  if (G_UNLIKELY(result->fontmap))
-    return result;
-  g_weak_ref_set ((GWeakRef *)&result->fontmap, fontmap);
+  g_assert (result->fontmap == NULL);
+  result->fontmap = fontmap;
+  g_object_add_weak_pointer (G_OBJECT (result->fontmap), (gpointer *) (gpointer) &result->fontmap);
 
   result->size = size;
   _pango_win32_make_matching_logfontw (fontmap, lfp, size, &result->logfontw);
@@ -1160,11 +1159,26 @@ pango_win32_font_description_from_logfont (const LOGFONT *lfp)
 
   variant = PANGO_VARIANT_NORMAL;
 
+  /* The PangoWeight values PANGO_WEIGHT_* map exactly do Windows FW_*
+   * values.  Is this on purpose? Quantize the weight to exact
+   * PANGO_WEIGHT_* values. Is this a good idea?
+   */
   if (lfp->lfWeight == FW_DONTCARE)
     weight = PANGO_WEIGHT_NORMAL;
+  else if (lfp->lfWeight <= (FW_ULTRALIGHT + FW_LIGHT) / 2)
+    weight = PANGO_WEIGHT_ULTRALIGHT;
+  else if (lfp->lfWeight <= (FW_LIGHT + FW_NORMAL) / 2)
+    weight = PANGO_WEIGHT_LIGHT;
+  else if (lfp->lfWeight <= (FW_NORMAL + FW_SEMIBOLD) / 2)
+    weight = PANGO_WEIGHT_NORMAL;
+  else if (lfp->lfWeight <= (FW_SEMIBOLD + FW_BOLD) / 2)
+    weight = PANGO_WEIGHT_SEMIBOLD;
+  else if (lfp->lfWeight <= (FW_BOLD + FW_ULTRABOLD) / 2)
+    weight = PANGO_WEIGHT_BOLD;
+  else if (lfp->lfWeight <= (FW_ULTRABOLD + FW_HEAVY) / 2)
+    weight = PANGO_WEIGHT_ULTRABOLD;
   else
-    /* The PangoWeight values PANGO_WEIGHT_* map exactly to Windows FW_*. */
-    weight = (PangoWeight) lfp->lfWeight;
+    weight = PANGO_WEIGHT_HEAVY;
 
   /* XXX No idea how to figure out the stretch */
   stretch = PANGO_STRETCH_NORMAL;
@@ -1345,11 +1359,26 @@ pango_win32_font_description_from_logfontw (const LOGFONTW *lfp)
 
   variant = PANGO_VARIANT_NORMAL;
 
+  /* The PangoWeight values PANGO_WEIGHT_* map exactly do Windows FW_*
+   * values.  Is this on purpose? Quantize the weight to exact
+   * PANGO_WEIGHT_* values. Is this a good idea?
+   */
   if (lfp->lfWeight == FW_DONTCARE)
     weight = PANGO_WEIGHT_NORMAL;
+  else if (lfp->lfWeight <= (FW_ULTRALIGHT + FW_LIGHT) / 2)
+    weight = PANGO_WEIGHT_ULTRALIGHT;
+  else if (lfp->lfWeight <= (FW_LIGHT + FW_NORMAL) / 2)
+    weight = PANGO_WEIGHT_LIGHT;
+  else if (lfp->lfWeight <= (FW_NORMAL + FW_SEMIBOLD) / 2)
+    weight = PANGO_WEIGHT_NORMAL;
+  else if (lfp->lfWeight <= (FW_SEMIBOLD + FW_BOLD) / 2)
+    weight = PANGO_WEIGHT_SEMIBOLD;
+  else if (lfp->lfWeight <= (FW_BOLD + FW_ULTRABOLD) / 2)
+    weight = PANGO_WEIGHT_BOLD;
+  else if (lfp->lfWeight <= (FW_ULTRABOLD + FW_HEAVY) / 2)
+    weight = PANGO_WEIGHT_ULTRABOLD;
   else
-    /* The PangoWeight values PANGO_WEIGHT_* map exactly to Windows FW_*. */
-    weight = (PangoWeight) lfp->lfWeight;
+    weight = PANGO_WEIGHT_HEAVY;
 
   /* XXX No idea how to figure out the stretch */
   stretch = PANGO_STRETCH_NORMAL;
@@ -1479,8 +1508,6 @@ pango_win32_insert_font (PangoWin32FontMap *win32fontmap,
   for (i = 0; i < PANGO_WIN32_N_COVERAGES; i++)
      win32face->coverages[i] = NULL;
 
-  win32face->face_name = NULL;
-
   win32face->is_synthetic = is_synthetic;
 
   win32face->has_cmap = TRUE;
@@ -1567,47 +1594,15 @@ pango_win32_face_is_synthesized (PangoFontFace *face)
   return win32face->is_synthetic;
 }
 
-G_DEFINE_TYPE (PangoWin32Face, pango_win32_face, PANGO_TYPE_FONT_FACE)
-
-static void
-pango_win32_face_finalize (GObject *object)
-{
-  int j;
-  PangoWin32Face *win32face = PANGO_WIN32_FACE (object);
-
-  pango_font_description_free (win32face->description);
-
-  for (j = 0; j < PANGO_WIN32_N_COVERAGES; j++)
-    if (win32face->coverages[j] != NULL)
-      pango_coverage_unref (win32face->coverages[j]);
-
-  g_free (win32face->face_name);
-
-  //g_free (win32face->cmap); // Err, cmap does not have lifecycle management currently :(
-
-  g_slist_free (win32face->cached_fonts);
-//  g_slist_free_full (win32face->cached_fonts, g_object_unref); // This doesn't work.
-
-  G_OBJECT_CLASS (pango_win32_family_parent_class)->finalize (object);
-}
-
 static void
 pango_win32_face_class_init (PangoFontFaceClass *class)
 {
-  GObjectClass *object_class = G_OBJECT_CLASS (class);
-
-  object_class->finalize = pango_win32_face_finalize;
   class->describe = pango_win32_face_describe;
   class->get_face_name = pango_win32_face_get_face_name;
   class->list_sizes = pango_win32_face_list_sizes;
   class->is_synthesized = pango_win32_face_is_synthesized;
 }
 
-static void
-pango_win32_face_init (PangoWin32Face *face)
-{
-}
-
 static void
 pango_win32_face_list_sizes (PangoFontFace  *face,
 			     int           **sizes,
@@ -1621,9 +1616,37 @@ pango_win32_face_list_sizes (PangoFontFace  *face,
   *n_sizes = 0;
 }
 
+static GType
+pango_win32_face_get_type (void)
+{
+  static GType object_type = 0;
+
+  if (G_UNLIKELY (!object_type))
+    {
+      static const GTypeInfo object_info =
+      {
+        sizeof (PangoFontFaceClass),
+        (GBaseInitFunc) NULL,
+        (GBaseFinalizeFunc) NULL,
+        (GClassInitFunc) pango_win32_face_class_init,
+        NULL,           /* class_finalize */
+        NULL,           /* class_data */
+        sizeof (PangoWin32Face),
+        0,              /* n_preallocs */
+        (GInstanceInitFunc) NULL,
+      };
+      
+      object_type = g_type_register_static (PANGO_TYPE_FONT_FACE,
+                                            I_("PangoWin32Face"),
+                                            &object_info, 0);
+    }
+  
+  return object_type;
+}
+
 /**
  * pango_win32_font_map_get_font_cache:
- * @font_map: a <type>PangoWin32FontMap</type>.
+ * @font_map: a #PangoWin32FontMap.
  *
  * Obtains the font cache associated with the given font map.
  *
@@ -1632,9 +1655,7 @@ pango_win32_face_list_sizes (PangoFontFace  *face,
 PangoWin32FontCache *
 pango_win32_font_map_get_font_cache (PangoFontMap *font_map)
 {
-  if (G_UNLIKELY (!font_map))
-    return NULL;
-
+  g_return_val_if_fail (font_map != NULL, NULL);
   g_return_val_if_fail (PANGO_WIN32_IS_FONT_MAP (font_map), NULL);
 
   return PANGO_WIN32_FONT_MAP (font_map)->font_cache;
@@ -1661,118 +1682,3 @@ pango_win32_fontmap_cache_clear (PangoWin32FontMap *win32fontmap)
   g_queue_free (win32fontmap->freed_fonts);
   win32fontmap->freed_fonts = g_queue_new ();
 }
-
-static PangoFontset *
-pango_win32_font_map_load_fontset (PangoFontMap                 *fontmap,
-				PangoContext                 *context,
-				const PangoFontDescription   *desc,
-				PangoLanguage                *language)
-{
-  /* This "adds" a load_fontset() for the Win32 backend */
-  /* which is needed to make sure we use an appropriate */
-  /* font for various texts when we are on Windows */
-  /* (Copied directly from pango-fontmap.c) */
-  PangoFontDescription *tmp_desc = pango_font_description_copy_static (desc);
-  const char *family;
-  char **families;
-  int i;
-  PangoFontsetSimple *fonts;
-  static GHashTable *warned_fonts = NULL; /* MT-safe */
-  G_LOCK_DEFINE_STATIC (warned_fonts);
-
-  g_return_val_if_fail (fontmap != NULL, NULL);
-
-  family = pango_font_description_get_family (desc);
-  families = g_strsplit (family ? family : "", ",", -1);
-
-  fonts = pango_fontset_simple_new (language);
-
-  for (i = 0; families[i]; i++)
-    pango_win32_font_map_fontset_add_fonts (fontmap,
-				      context,
-				      fonts,
-				      tmp_desc,
-				      families[i]);
-
-  g_strfreev (families);
-
-  /* The font description was completely unloadable, try with
-   * family == "Sans"
-   */
-  if (pango_fontset_simple_size (fonts) == 0)
-    {
-      char *ctmp1, *ctmp2;
-
-      pango_font_description_set_family_static (tmp_desc,
-						pango_font_description_get_family (desc));
-
-      ctmp1 = pango_font_description_to_string (desc);
-      pango_font_description_set_family_static (tmp_desc, "Sans");
-
-      G_LOCK (warned_fonts);
-      if (!warned_fonts || !g_hash_table_lookup (warned_fonts, ctmp1))
-	{
-	  if (!warned_fonts)
-	    warned_fonts = g_hash_table_new (g_str_hash, g_str_equal);
-
-	  g_hash_table_insert (warned_fonts, g_strdup (ctmp1), GINT_TO_POINTER (1));
-
-	  ctmp2 = pango_font_description_to_string (tmp_desc);
-	  g_warning ("couldn't load font \"%s\", falling back to \"%s\", "
-		     "expect ugly output.", ctmp1, ctmp2);
-	  g_free (ctmp2);
-	}
-      G_UNLOCK (warned_fonts);
-      g_free (ctmp1);
-
-      pango_win32_font_map_fontset_add_fonts (fontmap,
-					context,
-					fonts,
-					tmp_desc,
-					"Sans");
-    }
-
-  /* We couldn't try with Sans and the specified style. Try Sans Normal
-   */
-  if (pango_fontset_simple_size (fonts) == 0)
-    {
-      char *ctmp1, *ctmp2;
-
-      pango_font_description_set_family_static (tmp_desc, "Sans");
-      ctmp1 = pango_font_description_to_string (tmp_desc);
-      pango_font_description_set_style (tmp_desc, PANGO_STYLE_NORMAL);
-      pango_font_description_set_weight (tmp_desc, PANGO_WEIGHT_NORMAL);
-      pango_font_description_set_variant (tmp_desc, PANGO_VARIANT_NORMAL);
-      pango_font_description_set_stretch (tmp_desc, PANGO_STRETCH_NORMAL);
-
-      G_LOCK (warned_fonts);
-      if (!warned_fonts || !g_hash_table_lookup (warned_fonts, ctmp1))
-	{
-	  g_hash_table_insert (warned_fonts, g_strdup (ctmp1), GINT_TO_POINTER (1));
-
-	  ctmp2 = pango_font_description_to_string (tmp_desc);
-
-	  g_warning ("couldn't load font \"%s\", falling back to \"%s\", "
-		     "expect ugly output.", ctmp1, ctmp2);
-	  g_free (ctmp2);
-	}
-      G_UNLOCK (warned_fonts);
-      g_free (ctmp1);
-
-      pango_win32_font_map_fontset_add_fonts (fontmap,
-					context,
-					fonts,
-					tmp_desc,
-					"Sans");
-    }
-
-  pango_font_description_free (tmp_desc);
-
-  /* Everything failed, we are screwed, there is no way to continue,
-   * but lets just not crash here.
-   */
-  if (pango_fontset_simple_size (fonts) == 0)
-      g_warning ("All font fallbacks failed!!!!");
-
-  return PANGO_FONTSET (fonts);
-}
